# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path('../../../test_helper', __FILE__)

class Mmtrix::Agent::RpmAgentTest < Minitest::Test
  def setup
    Mmtrix::Agent.manual_start
    @agent = Mmtrix::Agent.instance
    @agent.stubs(:start_worker_thread)
  end

  def teardown
    Mmtrix::Agent.instance.shutdown
  end

  def test_public_apis
    assert_raises(RuntimeError) do
      Mmtrix::Agent.set_sql_obfuscator(:unknown) { |sql| puts sql }
    end

    ignore_called = false
    filter = Proc.new do |e|
      ignore_called = true
      nil
    end
    with_ignore_error_filter(filter) do
      Mmtrix::Agent.notice_error(StandardError.new("message"), :request_params => {:x => "y"})
    end

    assert(ignore_called)
  end

  def test_startup_shutdown_real
    with_config(:agent_enabled => true, :monitor_mode => true) do
      Mmtrix::Agent.manual_start :monitor_mode => true, :license_key => ('x' * 40)
      agent = Mmtrix::Agent.instance
      assert agent.started?
      agent.shutdown
      refute agent.started?
    end
  end

  def test_manual_start
    Mmtrix::Agent.instance.expects(:connect).once
    Mmtrix::Agent.instance.expects(:start_worker_thread).once
    Mmtrix::Agent.instance.instance_variable_set '@started', nil
    Mmtrix::Agent.manual_start :monitor_mode => true, :license_key => ('x' * 40)
    Mmtrix::Agent.shutdown
  end

  def test_post_fork_handler
    Mmtrix::Agent.manual_start :monitor_mode => true, :license_key => ('x' * 40)
    Mmtrix::Agent.after_fork
    Mmtrix::Agent.after_fork
    Mmtrix::Agent.shutdown
  end

  def test_manual_overrides
    Mmtrix::Agent.manual_start :app_name => "testjobs"
    assert_equal "testjobs", Mmtrix::Agent.config.app_names[0]
    Mmtrix::Agent.shutdown
  end

  def test_agent_restart
    Mmtrix::Agent.manual_start :app_name => "noapp"
    Mmtrix::Agent.manual_start :app_name => "testjobs"
    assert_equal "testjobs", Mmtrix::Agent.config.app_names[0]
    Mmtrix::Agent.shutdown
  end

  def test_set_record_sql
    @agent.set_record_sql(false)
    assert !Mmtrix::Agent.tl_is_sql_recorded?
    Mmtrix::Agent.disable_sql_recording do
      assert_equal false, Mmtrix::Agent.tl_is_sql_recorded?
      Mmtrix::Agent.disable_sql_recording do
        assert_equal false, Mmtrix::Agent.tl_is_sql_recorded?
      end
      assert_equal false, Mmtrix::Agent.tl_is_sql_recorded?
    end
    assert !Mmtrix::Agent.tl_is_sql_recorded?
    @agent.set_record_sql(nil)
  end

  def test_agent_version_string
    assert_match(/\d\.\d+\.\d+/, Mmtrix::VERSION::STRING)
  end

  def test_record_transaction
    Mmtrix::Agent.record_transaction 0.5, 'uri' => "/users/create?foo=bar"
  end
end
