# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__), '..', '..', 'test_helper'))
require 'mmtrix/control/instance_methods'
require 'mmtrix/agent/configuration/yaml_source'

class TestClass
  include Mmtrix::Control::InstanceMethods
end

class Mmtrix::Control::InstanceMethodsTest < Minitest::Test
  def setup
    Mmtrix::Agent.config.reset_to_defaults
    @test = ::TestClass.new(nil)
  end

  def test_configure_agent_adds_the_yaml_config
    refute_has_config Mmtrix::Agent::Configuration::YamlSource
    @test.configure_agent('test', {})
    assert_has_config Mmtrix::Agent::Configuration::YamlSource
  end

  def test_configure_agent_adds_the_manual_config
    refute_has_config Mmtrix::Agent::Configuration::ManualSource
    @test.configure_agent('test', {})
    assert_has_config Mmtrix::Agent::Configuration::ManualSource
  end

  def test_no_high_security_config_by_default
    refute_has_config Mmtrix::Agent::Configuration::HighSecuritySource
    @test.configure_agent('test', {:high_security => false})
    refute_has_config Mmtrix::Agent::Configuration::HighSecuritySource
  end

  def test_high_security_config_added_if_requested
    refute_has_config Mmtrix::Agent::Configuration::HighSecuritySource
    @test.configure_agent('test', {:high_security => true})
    assert_has_config Mmtrix::Agent::Configuration::HighSecuritySource
  end

  def refute_has_config(clazz)
    refute Mmtrix::Agent.config.config_classes_for_testing.include? clazz
  end

  def assert_has_config(clazz)
    assert Mmtrix::Agent.config.config_classes_for_testing.include? clazz
  end
end
