# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__),'/../test_helper'))

class Mmtrix::ControlTest < Minitest::Test
  attr_reader :control

  def setup
    @control =  Mmtrix::Control.instance
    raise 'oh geez, wrong class' unless Mmtrix::Control.instance.is_a?(::Mmtrix::Control::Frameworks::Test)
    Mmtrix::Agent.config.reset_to_defaults
  end

  def shutdown
    Mmtrix::Agent.shutdown
  end

  def test_settings_accessor
    refute_nil control.settings
  end

  def test_root
    assert File.directory?(Mmtrix::Control.mmtrix_root), Mmtrix::Control.mmtrix_root
    if defined?(Rails::VERSION)
      assert File.directory?(File.join(Mmtrix::Control.mmtrix_root, "lib")), Mmtrix::Control.mmtrix_root +  "/lib"
    end
  end

  def test_info
    Mmtrix::Agent.manual_start
    if defined?(Rails::VERSION)
      assert_match(/jdbc|postgres|mysql|sqlite/, Mmtrix::EnvironmentReport.new["Database adapter"])
    end
  end

  def test_api_server_uses_configured_values
    control.instance_variable_set(:@api_server, nil)
    with_config(:api_host => 'somewhere', :api_port => 8080) do
      assert_equal 'somewhere', control.api_server.name
      assert_equal 8080, control.api_server.port
    end
  end

  def test_server_from_host_uses_configured_values
    with_config(:host => 'donkeytown', :port => 8080) do
      assert_equal 'donkeytown', control.server_from_host.name
      assert_equal 8080, control.server_from_host.port
    end
  end

  def test_transaction_threshold__override
    with_config(:transaction_tracer => { :transaction_threshold => 1}) do
      assert_equal 1, Mmtrix::Agent.config[:'transaction_tracer.transaction_threshold']
    end
  end

  def test_transaction_tracer_disabled
    with_config(:'transaction_tracer.enabled' => false,
                :developer_mode => false, :monitor_mode => true) do
      assert(!Mmtrix::Agent.instance.transaction_sampler.enabled?,
             'transaction tracer enabled when config calls for disabled')
    end
  end

  def test_sql_tracer_disabled
    with_config(:'slow_sql.enabled' => false, :monitor_mode => true) do
      assert(!Mmtrix::Agent.instance.sql_sampler.enabled?,
             'sql tracer enabled when config calls for disabled')
    end
  end

  def test_sql_tracer_disabled_with_record_sql_false
    with_config(:slow_sql => { :enabled => true, :record_sql => 'off' }) do
      refute Mmtrix::Agent.instance.sql_sampler.enabled?,
             'sql tracer enabled when config calls for disabled'
    end
  end

  def test_sql_tracer_disabled_when_tt_disabled
    with_config(:'transaction_tracer.enabled' => false,
                :'slow_sql.enabled' => true,
                :developer_mode => false, :monitor_mode => true) do
      refute Mmtrix::Agent.instance.sql_sampler.enabled?,
             'sql enabled when transaction tracer disabled'
    end
  end

  def test_sql_tracer_disabled_when_tt_disabled_by_server
    with_config_low_priority({
                 :'slow_sql.enabled'           => true,
                 :'transaction_tracer.enabled' => true,
                 :monitor_mode                 => true}) do
      Mmtrix::Agent.instance.finish_setup('collect_traces' => false)

      refute Mmtrix::Agent::Agent.instance.sql_sampler.enabled?,
             'sql enabled when tracing disabled by server'
    end
  end

  def test_init_plugin_loads_samplers_enabled
    reset_agent

    with_config(:disable_samplers       => false,
                :disable_harvest_thread => true,
                :agent_enabled          => true,
                :monitor_mode           => true,
                :license_key            => 'a'*40) do
      Mmtrix::Control.instance.init_plugin
      assert Mmtrix::Agent.instance.harvest_samplers.any?
    end
  end

  def test_init_plugin_loads_samplers_disabled
    reset_agent

    with_config(:disable_samplers       => true,
                :disable_harvest_thread => true,
                :agent_enabled          => true,
                :monitor_mode           => true,
                :license_key            => 'a'*40) do
      Mmtrix::Control.instance.init_plugin
      refute Mmtrix::Agent.instance.harvest_samplers.any?
    end
  end

  def test_agent_not_starting_does_not_load_samplers
    reset_agent

    Mmtrix::Agent.instance.stubs(:defer_for_delayed_job?).returns(true)

    with_config(:disable_samplers       => false,
                :disable_harvest_thread => true,
                :agent_enabled          => true,
                :monitor_mode           => true,
                :license_key            => 'a'*40) do
      Mmtrix::Control.instance.init_plugin
      refute Mmtrix::Agent.instance.already_started?
      refute Mmtrix::Agent.instance.harvest_samplers.any?
    end
  end

  def test_agent_starting_after_fork_does_load_samplers
    reset_agent

    Mmtrix::Agent.instance.stubs(:defer_for_delayed_job?).returns(true)

    with_config(:disable_samplers       => false,
                :disable_harvest_thread => true,
                :agent_enabled          => true,
                :monitor_mode           => true,
                :license_key            => 'a'*40) do
      Mmtrix::Control.instance.init_plugin
      Mmtrix::Agent.instance.stubs(:defer_for_delayed_job?).returns(false)
      Mmtrix::Agent.after_fork
      assert Mmtrix::Agent.instance.already_started?
      assert Mmtrix::Agent.instance.harvest_samplers.any?
    end
  end

  def reset_agent
    Mmtrix::Agent.shutdown
    Mmtrix::Agent.instance.harvest_samplers.clear
    Mmtrix::Agent.instance.instance_variable_set(:@connect_state, :pending)
    Mmtrix::Agent.instance.instance_variable_set(:@worker_thread, nil)
    Mmtrix::Agent.instance.harvester.instance_variable_set(:@starting_pid, nil)
  end
end
