# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__),'..', 'test_helper'))

# Test logic around detecting or configuring dispatcher
class DispatcherTest < Minitest::Test

  def setup
    Mmtrix::Agent.shutdown
    Mmtrix::Agent.reset_config
  end

  def teardown
    Mmtrix::Agent.reset_config
    Mmtrix::Control.reset
  end

  def assert_dispatcher_reported_to_environment_report(dispatcher)
    assert_equal dispatcher.to_s, Mmtrix::EnvironmentReport.new["Dispatcher"]
  end

  def test_detects_dispatcher_via_loaded_libraries
    class << self
      module ::PhusionPassenger
      end
    end
    assert_equal :passenger, Mmtrix::Agent.config[:dispatcher]
    assert_dispatcher_reported_to_environment_report :passenger
  ensure
    Object.send(:remove_const, :PhusionPassenger)
  end

  def test_detects_dispatcher_via_ENV_MM_TRIX_DISPATCHER
    ENV['MM_TRIX_DISPATCHER'] = "foobared"
    Mmtrix::Agent.reset_config
    assert_equal :foobared, Mmtrix::Agent.config[:dispatcher]
    assert_dispatcher_reported_to_environment_report :foobared
  ensure
    ENV['MM_TRIX_DISPATCHER'] = nil
  end

  def test_detects_dispatcher_via_ENV_MMTRIX_DISPATCHER
    ENV['MMTRIX_DISPATCHER'] = "bazbang"
    Mmtrix::Agent.reset_config
    assert_equal :bazbang, Mmtrix::Agent.config[:dispatcher]
    assert_dispatcher_reported_to_environment_report :bazbang
  ensure
    ENV['MMTRIX_DISPATCHER'] = nil
  end

  def test_detects_dispatcher_based_on_arguments_to_manual_start
    Mmtrix::Agent.manual_start(:dispatcher   => :resque)
    assert_equal :resque, Mmtrix::Agent.config[:dispatcher]
    assert_dispatcher_reported_to_environment_report :resque
  end

end
