# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__),'..', 'test_helper'))

# Test logic around detecting or configuring framework
class FrameworkTest < Minitest::Test

  def setup

    # muck with this constant which forces the agent to load the
    # Mmtrix::Control::Frameworks::Test control so we can test the logic used
    # to load the appropriate control object.
    @old_mmtrix_test_const = ::Mmtrix::TEST
    ::Mmtrix.send(:remove_const, :TEST)

    Mmtrix::Agent.shutdown
    Mmtrix::Agent.reset_config

    # don't bomb out trying to load frameworks that don't exist.
    Mmtrix::Control.stubs(:new_instance).returns(stub :init_plugin => nil)
  end

  def teardown
    # Put things back how we found them
    ::Mmtrix.send(:const_set, :TEST,  @old_mmtrix_test_const)
    Mmtrix::Agent.reset_config
  end

  def test_detects_framework_via_loaded_libraries
    class << self
      module ::Merb
        module Plugins
        end
      end
    end
    assert_equal :merb, Mmtrix::Agent.config[:framework]
  ensure
    Object.send(:remove_const, :Merb)
  end

  def test_detects_framework_via_ENV_MM_TRIX_FRAMEWORK
    ENV['MM_TRIX_FRAMEWORK'] = "foobared"
    Mmtrix::Agent.reset_config
    assert_equal :foobared, Mmtrix::Agent.config[:framework]
  ensure
    ENV['MM_TRIX_FRAMEWORK'] = nil
  end

  def test_detects_framework_via_ENV_MMTRIX_FRAMEWORK
    ENV['MMTRIX_FRAMEWORK'] = "bazbang"
    Mmtrix::Agent.reset_config
    assert_equal :bazbang, Mmtrix::Agent.config[:framework]
  ensure
    ENV['MMTRIX_FRAMEWORK'] = nil
  end
end
