# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__),'..','..','test_helper'))
require 'mmtrix/rack/agent_hooks'

class AgentHooksTest < Minitest::Test

  def setup
    @app = stub_everything
    @hooks = Mmtrix::Rack::AgentHooks.new(@app)
    @env = {:env => "env"}
  end

  def test_before_call
    Mmtrix::Agent.instance.events.expects(:notify).with(:start_transaction)
    Mmtrix::Agent.instance.events.expects(:notify).with(:before_call, @env)
    Mmtrix::Agent.instance.events.stubs(:notify).with(:after_call, anything, anything)
    Mmtrix::Agent.instance.events.expects(:notify).with(:transaction_finished, anything)

    @hooks.call(@env)
  end

  def test_after_call
    result = [stub, stub, stub]
    @app.stubs(:call).returns(result)

    Mmtrix::Agent.instance.events.expects(:notify).with(:start_transaction)
    Mmtrix::Agent.instance.events.stubs(:notify).with(:before_call, anything)
    Mmtrix::Agent.instance.events.expects(:notify).with(:after_call, @env, result)
    Mmtrix::Agent.instance.events.expects(:notify).with(:transaction_finished, anything)

    @hooks.call(@env)
  end

  def test_nested_agent_hooks_still_fire_only_once
    nested = Mmtrix::Rack::AgentHooks.new(@hooks)

    Mmtrix::Agent.instance.events.expects(:notify).times(4)
    nested.call(@env)
  end

end
