# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require File.expand_path(File.join(File.dirname(__FILE__),'..', '..',
                                   'test_helper'))
require 'rack/test'
require 'mmtrix/rack/error_collector'

module Mmtrix::Rack
  class ErrorCollectorTest < Minitest::Test
    include Rack::Test::Methods

    class TestApp
      def call(env)
        raise 'unhandled error'
      end
    end

    def app
      Mmtrix::Rack::ErrorCollector.new(TestApp.new)
    end

    def setup
      Mmtrix::Agent.reset_config
      Mmtrix::Agent.manual_start
      Mmtrix::Agent.instance.error_collector
      Mmtrix::Agent.instance.error_collector.errors = []

      # sanity checks
      assert Mmtrix::Agent.instance.error_collector.enabled?
    end

    def test_notice_and_reraise_errors
      assert_raises RuntimeError do
        get '/'
      end

      assert_equal('unhandled error', last_error.message)
    end

    def test_ignore_filtered_errors
      filter = Proc.new do |error|
        !error.kind_of?(RuntimeError)
      end

      with_ignore_error_filter(filter) do
        assert_raises RuntimeError do
          get '/'
        end
      end

      assert(Mmtrix::Agent.instance.error_collector.errors.empty?,
             'noticed an error that should have been ignored')
    end

    # Ideally we'd test this for failures to create Rack::Request as well,
    # but unfortunately rack-test, which we're using to drive, creates
    # Rack::Request objects internally, so there's not an easy way to.
    def test_handles_failure_to_create_request_object
      if defined?(ActionDispatch::Request)
        ActionDispatch::Request.stubs(:new).raises('bad news')

        assert_raises RuntimeError do
          get '/foo/bar?q=12'
        end

        assert_equal('unhandled error', last_error.message)
        assert_equal('/foo/bar', last_error.request_uri)
      end
    end

    def last_error
      Mmtrix::Agent.instance.error_collector.errors[0]
    end
  end
end
