# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require 'securerandom'

class AgentAttributesTests < Performance::TestCase
  def setup
    require 'mmtrix/agent/attribute_filter'
  end

  ALPHA = "alpha".freeze
  BETA  = "beta".freeze

  def test_empty_agent_attributes
    @filter = Mmtrix::Agent::AttributeFilter.new(Mmtrix::Agent.config)

    measure do
      @filter.apply(ALPHA, Mmtrix::Agent::AttributeFilter::DST_ALL)
      @filter.apply(BETA, Mmtrix::Agent::AttributeFilter::DST_ALL)
    end
  end

  def test_with_attribute_rules
    with_config(:'attributes.include' => ['alpha'],
                :'attributes.exclude' => ['beta']) do

      @filter = Mmtrix::Agent::AttributeFilter.new(Mmtrix::Agent.config)

      measure do
        @filter.apply(ALPHA, Mmtrix::Agent::AttributeFilter::DST_ALL)
        @filter.apply(BETA, Mmtrix::Agent::AttributeFilter::DST_ALL)
      end
    end
  end

  def test_with_wildcards
    with_config(:'attributes.include' => ['alpha*'],
                :'attributes.exclude' => ['beta*']) do

      @filter = Mmtrix::Agent::AttributeFilter.new(Mmtrix::Agent.config)

      measure do
        @filter.apply(ALPHA, Mmtrix::Agent::AttributeFilter::DST_ALL)
        @filter.apply(BETA, Mmtrix::Agent::AttributeFilter::DST_ALL)
      end
    end
  end

  def test_with_tons_o_rules
    with_config(:'attributes.include' => 100.times.map { SecureRandom.hex },
                :'attributes.exclude' => 100.times.map { SecureRandom.hex }) do

      @filter = Mmtrix::Agent::AttributeFilter.new(Mmtrix::Agent.config)

      measure do
        @filter.apply(ALPHA, Mmtrix::Agent::AttributeFilter::DST_ALL)
        @filter.apply(BETA, Mmtrix::Agent::AttributeFilter::DST_ALL)
      end
    end
  end
end
